<section id="containers-plugin" class="plugin" ng-if="vm.containers.length">
    <span class="title">CONTAINERS</span> {{ vm.containers.length }} (served by Docker {{ vm.version }})

    <div class="table">
        <div class="table-row">
            <div class="table-cell text-left">Name</div>
            <div class="table-cell">Status</div>
            <div class="table-cell">Uptime</div>
            <div class="table-cell">CPU%</div>
            <div class="table-cell">MEM</div>
            <div class="table-cell">RSS</div>
            <div class="table-cell">IOR/s</div>
            <div class="table-cell">IOW/s</div>
            <div class="table-cell">RX/s</div>
            <div class="table-cell">TX/s</div>
            <div class="table-cell text-left">Command</div>
        </div>
        <div class="table-row" ng-repeat="container in vm.containers track by container.id">
            <div class="table-cell text-left">{{ container.name }}</div>
            <div class="table-cell" ng-class="container.status == 'Paused' ? 'careful' : 'ok'">{{ container.status }}
            </div>
            <div class="table-cell" ng-class="container.status == 'Paused' ? 'careful' : 'ok'">{{ container.uptime }}
            </div>
            <div class="table-cell">{{ container.cpu | number:1 }}</div>
            <div class="table-cell">{{ container.memory | bytes }}</div>
            <div class="table-cell">{{ container.rss | bytes }}</div>
            <div class="table-cell">{{ container.ior / container.io_time_since_update | bits }}</div>
            <div class="table-cell">{{ container.iow / container.io_time_since_update | bits }}</div>
            <div class="table-cell">{{ container.rx / container.net_time_since_update | bits }}</div>
            <div class="table-cell">{{ container.tx / container.net_time_since_update | bits }}</div>
            <div class="table-cell text-left">{{ container.command }}</div>
        </div>
    </div>
</section>
