<section id="gpu" class="plugin">
    <div class="gpu-name title">
        {{ vm.name }}
    </div>
    <div class="table">
        <div class="table-row" ng-if="arguments.meangpu || vm.gpus.length === 1">
            <div class="table-cell text-left">proc:</div>
            <div class="table-cell" ng-class="vm.getMeanDecoration('proc')" ng-if="vm.mean.proc != null">{{ vm.mean.proc |
                number : 0 }}%
            </div>
            <div class="table-cell" ng-if="vm.mean.proc == null">N/A</div>
        </div>
        <div class="table-row" ng-if="arguments.meangpu || vm.gpus.length === 1">
            <div class="table-cell text-left">mem:</div>
            <div class="table-cell" ng-class="vm.getMeanDecoration('mem')" ng-if="vm.mean.mem != null">{{ vm.mean.mem | number :
                0 }}%
            </div>
            <div class="table-cell" ng-if="vm.mean.mem == null">N/A</div>
        </div>
        <div class="table-row" ng-if="arguments.meangpu || vm.gpus.length === 1">
            <div class="table-cell text-left">temperature::</div>
            <div class="table-cell" ng-class="vm.getMeanDecoration('temperature')" ng-if="vm.mean.temperature != null">{{ vm.mean.temperature | number
                :
                0 }}°
            </div>
            <div class="table-cell" ng-if="vm.mean.temperature == null">N/A</div>
        </div>
        <div class="table-row" ng-if="!arguments.meangpu && vm.gpus.length > 1" ng-repeat="gpu in vm.gpus">
            <div class="table-cell text-left">
                {{ gpu.gpu_id }}:
                <span ng-class="vm.getDecoration(gpu.gpu_id, 'proc')" ng-if="gpu.proc != null">{{ gpu.proc | number : 0 }}%</span>
                <span ng-if="gpu.proc == null">N/A</span>
                mem:
                <span ng-class="vm.getDecoration(gpu.gpu_id, 'mem')" ng-if="gpu.mem != null">{{ gpu.mem | number : 0 }}%</span>
                <span ng-if="gpu.mem == null">N/A</span>
                temp:
                <span ng-class="vm.getDecoration(gpu.gpu_id, 'temperature')" ng-if="gpu.temperature != null">{{ gpu.temperature | number : 0 }}C</span>
                <span ng-if="gpu.temperature == null">N/A</span>
            </div>
        </div>
    </div>
</section>
