<section id="processlist-plugin" class="plugin">
    <div class="table">
        <div class="table-row">
            <div sortable-th sorter="vm.sorter" column="cpu_percent" class="table-cell">CPU%</div>
            <div sortable-th sorter="vm.sorter" column="memory_percent" class="table-cell">MEM%</div>
            <div class="table-cell hidden-xs hidden-sm">VIRT</div>
            <div class="table-cell hidden-xs hidden-sm">RES</div>
            <div class="table-cell">PID</div>
            <div sortable-th sorter="vm.sorter" column="username" class="table-cell text-left">USER</div>
            <div sortable-th sorter="vm.sorter" column="timemillis" class="table-cell hidden-xs hidden-sm">TIME+</div>
            <div sortable-th sorter="vm.sorter" column="num_threads" class="table-cell text-left hidden-xs hidden-sm">THR</div>
            <div class="table-cell">NI</div>
            <div class="table-cell">S</div>
            <div sortable-th sorter="vm.sorter" column="io_read" class="table-cell hidden-xs hidden-sm" ng-show="vm.ioReadWritePresent">IOR/s</div>
            <div sortable-th sorter="vm.sorter" column="io_write" class="table-cell text-left hidden-xs hidden-sm" ng-show="vm.ioReadWritePresent">IOW/s</div>
            <div sortable-th sorter="vm.sorter" column="name" class="table-cell text-left">Command</div>
        </div>
        <div class="table-row"
             ng-repeat="process in vm.processes | orderBy:vm.sorter.column:vm.sorter.isReverseColumn(vm.sorter.column) | limitTo: vm.getLimit() track by process.pid">
            <div class="table-cell" ng-class="vm.getCpuPercentAlert(process)">{{ process.cpu_percent == -1 ? '?' : (process.cpu_percent | number:1) }}</div>
            <div class="table-cell" ng-class="vm.getMemoryPercentAlert(process)">{{ process.memory_percent == -1 ? '?' : (process.memory_percent | number:1) }}</div>
            <div class="table-cell hidden-xs hidden-sm">{{process.memvirt | bytes}}</div>
            <div class="table-cell hidden-xs hidden-sm">{{process.memres | bytes}}</div>
            <div class="table-cell">{{process.pid}}</div>
            <div class="table-cell text-left">{{process.username}}</div>
            <div class="table-cell hidden-xs hidden-sm" ng-if="process.timeplus != '?'">
                <span ng-show="process.timeplus.hours > 0" class="highlight">{{ process.timeplus.hours }}h</span>{{
                process.timeplus.minutes | leftPad:2:'0' }}:{{ process.timeplus.seconds | leftPad:2:'0' }}<span
                    ng-show="process.timeplus.hours <= 0">.{{ process.timeplus.milliseconds | leftPad:2:'0' }}</span>
            </div>
            <div class="table-cell hidden-xs hidden-sm" ng-if="process.timeplus == '?'">?</div>
            <div class="table-cell text-left hidden-xs hidden-sm">{{ process.num_threads == -1 ? '?' : process.num_threads }}</div>
            <div class="table-cell" ng-class="{nice: process.isNice}">{{process.nice | exclamation}}</div>
            <div class="table-cell" ng-class="{status: process.status == 'R'}">{{process.status}}</div>
            <div class="table-cell hidden-xs hidden-sm" ng-show="vm.ioReadWritePresent">{{process.io_read | bytes}}</div>
            <div class="table-cell text-left hidden-xs hidden-sm" ng-show="vm.ioReadWritePresent">{{process.io_write | bytes}}</div>
            <div class="table-cell text-left" ng-show="vm.arguments.process_short_name">{{process.name}}</div>
            <div class="table-cell text-left" ng-show="!vm.arguments.process_short_name">{{process.cmdline}}</div>
        </div>
    </div>
</section>
