<section id="processcount" class="plugin">
    <span class="title">TASKS</span>
    <span>{{ vm.total }} ({{ vm.thread }} thr),</span>
    <span>{{ vm.running }} run,</span>
    <span>{{ vm.sleeping }} slp,</span>
    <span>{{ vm.stopped }} oth</span>
    <span> sorted {{ vm.sorter.auto ? 'automatically' : '' }} by {{ vm.sorter.getColumnLabel(vm.sorter.column) }}, flat view</span>
</section>